@@ -50,6 +50,7 @@ array set ::Configs [strip_comments {
     -O2
     --disable-amalgamation --disable-shared
     --enable-session
+    -DSQLITE_ENABLE_DESERIALIZE
   }
   "Sanitize" {
     CC=clang -fsanitize=undefined
@@ -80,6 +81,10 @@ array set ::Configs [strip_comments {
     -DSQLITE_THREADSAFE
     -DSQLITE_TCL_DEFAULT_FULLMUTEX=1
   }
+  "User-Auth" {
+    -O2
+    -DSQLITE_USER_AUTHENTICATION=1
+  }
   "Secure-Delete" {
     -O2
     -DSQLITE_SECURE_DELETE=1
@@ -127,6 +132,7 @@ array set ::Configs [strip_comments {
     -DSQLITE_ENABLE_HIDDEN_COLUMNS
     -DSQLITE_MAX_ATTACHED=125
     -DSQLITE_MUTATION_TEST
+    --enable-fts5 --enable-json1
   }
   "Fast-One" {
     -O6
@@ -267,11 +273,12 @@ array set ::Configs [strip_comments {
 array set ::Platforms [strip_comments {
   Linux-x86_64 {
     "Check-Symbols"           checksymbols
-    "Fast-One"                fuzztest
+    "Fast-One"                "fuzztest test"
     "Debug-One"               "mptest test"
     "Have-Not"                test
     "Secure-Delete"           test
     "Unlock-Notify"           "QUICKTEST_INCLUDE=notify2.test test"
+    "User-Auth"               tcltest
     "Update-Delete-Limit"     test
     "Extra-Robustness"        test
     "Device-Two"              test
